use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::EOL 0.18

use Test::More 0.88;
use Test::EOL;

my @files = (
    'lib/Test/Class/Moose.pm',
    'lib/Test/Class/Moose/AttributeRegistry.pm',
    'lib/Test/Class/Moose/Config.pm',
    'lib/Test/Class/Moose/Deprecated.pm',
    'lib/Test/Class/Moose/Executor/Parallel.pm',
    'lib/Test/Class/Moose/Executor/Sequential.pm',
    'lib/Test/Class/Moose/Load.pm',
    'lib/Test/Class/Moose/Report.pm',
    'lib/Test/Class/Moose/Report/Class.pm',
    'lib/Test/Class/Moose/Report/Instance.pm',
    'lib/Test/Class/Moose/Report/Method.pm',
    'lib/Test/Class/Moose/Report/Time.pm',
    'lib/Test/Class/Moose/Role.pm',
    'lib/Test/Class/Moose/Role/AutoUse.pm',
    'lib/Test/Class/Moose/Role/Executor.pm',
    'lib/Test/Class/Moose/Role/HasTimeReport.pm',
    'lib/Test/Class/Moose/Role/Parallel.pm',
    'lib/Test/Class/Moose/Role/ParameterizedInstances.pm',
    'lib/Test/Class/Moose/Role/Reporting.pm',
    'lib/Test/Class/Moose/Runner.pm',
    'lib/Test/Class/Moose/Tutorial.pm',
    't/00-report-prereqs.dd',
    't/00-report-prereqs.t',
    't/autouse.t',
    't/bare.t',
    't/basic.t',
    't/environment.t',
    't/include_classes.t',
    't/include_exclude.t',
    't/lib/PersonTest.pm',
    't/lib/TestsFor/Basic.pm',
    't/lib/TestsFor/Basic/Subclass.pm',
    't/load.t',
    't/loadlib/helpers/Fail.pm',
    't/loadlib/tests/LoadTests/Success.pm',
    't/non-randomize-classes.t',
    't/parallel-role.t',
    't/parallel.t',
    't/parallellib/TestsFor/Alpha.pm',
    't/parallellib/TestsFor/Alpha/Subclass.pm',
    't/parallellib/TestsFor/Beta.pm',
    't/parallellib/TestsFor/Sequential.pm',
    't/parameterized.t',
    't/parameterizedlib/TestsFor/Empty.pm',
    't/parameterizedlib/TestsFor/Parameterized.pm',
    't/planlib/TestsFor/Attributes.pm',
    't/planlib/TestsFor/Attributes/Subclass.pm',
    't/planlib/TestsFor/Person.pm',
    't/planlib/TestsFor/Person/Employee.pm',
    't/plans.t',
    't/process_name.t',
    't/processnamelib/TestsFor/Name1.pm',
    't/processnamelib/TestsFor/Parameterized.pm',
    't/randomlib/TestsFor/A.pm',
    't/randomlib/TestsFor/B.pm',
    't/randomlib/TestsFor/C.pm',
    't/randomlib/TestsFor/D.pm',
    't/randomlib/TestsFor/E.pm',
    't/reporting.t',
    't/reportpassed.t',
    't/reportpassedlib/TestsFor/Fail.pm',
    't/reportpassedlib/TestsFor/FailChild.pm',
    't/reportpassedlib/TestsFor/Pass.pm',
    't/skip.t',
    't/skip_all.t',
    't/skiplib/TestsFor/Basic.pm',
    't/skiplib/TestsFor/SkipSomeMethods.pm',
    't/taglib/PersonTest.pm',
    't/taglib/TestsFor/Basic.pm',
    't/taglib/TestsFor/Basic/Role.pm',
    't/taglib/TestsFor/Basic/Subclass.pm',
    't/taglib/TestsFor/Basic/WithRole.pm',
    't/taglib/TestsFor/MultipleExclude.pm',
    't/tags.t',
    't/test_control_methods.t'
);

eol_unix_ok($_, { trailing_whitespace => 1 }) foreach @files;
done_testing;
