use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007000
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib  ) );
__DATA__
drolsky
DROLSKY
DROLSKY's
PayPal
Rolsky
Rolsky's
API
AnnoCPAN
CPAN
DEPRECATIONS
Extensibility
MetaCPAN
OO
Refactor
STDOUT
Stuckdownawell
TCM
TODO
arisdottle
distro
hardcode
hashrefs
invocant
metadata
munge
munges
namespace
ok
online
parallelizable
parallelize
parallelized
parameterized
prereq
rebless
runtime
startup
strawman
subtest
subtests
teardown
toolkits
un
xUnit
Curtis
Ovid
Poe
ovid
Dave
autarch
Andy
Jack
github
Denny
de
la
Haye
denny
Doug
Bell
madcityzen
Gregory
Oschwald
goschwald
Jeremy
Krieg
Jonathan
Otsuka
djgoku
Stowe
jns
Karen
Etheridge
ether
mark
maflick88
mephinet
Neil
Bowers
neil
Olaf
Alders
olaf
Paul
Boyd
pboyd
Petrea
Corneliu
Stefan
stefan
Steven
Humphrey
catchgithub
stuckdownawell
Tim
Vroom
vroom
Tom
Beresford
tom
Heady
Udo
Oji
Velti
lib
Test
Class
Moose
AttributeRegistry
Config
Deprecated
Executor
Parallel
Sequential
Load
Report
Instance
Method
Time
Role
AutoUse
HasTimeReport
ParameterizedInstances
Reporting
Runner
Tutorial
